@@ -816,6 +816,44 @@ loadNewSqlCyclopsDataY <- function(object,
                       time)
 }
 
+#' @keywords internal
+loadNewSqlCyclopsDataStratTimeEffects <- function(object,
+                                                  stratumId = NULL,
+                                                  rowId = NULL,
+                                                  subjectId = NULL,
+                                                  timeEffectCovariateId) {
+
+    if (!isInitialized(object)) stop("Object is no longer or improperly initialized.")
+
+    if (is.unsorted(stratumId)) {
+        stop("All columns must be sorted first by stratumId (if supplied) and then by rowId")
+    }
+
+    if (!bit64::is.integer64(stratumId)) {
+        stratumId <- bit64::as.integer64(stratumId)
+    }
+
+    if (!bit64::is.integer64(rowId)) {
+        rowId <- bit64::as.integer64(rowId)
+    }
+
+    if (!bit64::is.integer64(subjectId)) {
+        subjectId <- bit64::as.integer64(subjectId)
+    }
+
+    if (!bit64::is.integer64(timeEffectCovariateId)) {
+        timeEffectCovariateId <- bit64::as.integer64(timeEffectCovariateId)
+    }
+
+    timeEffectCovariatesName <- .loadCyclopsDataStratTimeEffects(object,
+                                                                 stratumId,
+                                                                 rowId,
+                                                                 subjectId,
+                                                                 timeEffectCovariateId)
+    if (!is.null(object$coefficientNames)) {
+        object$coefficientNames <- append(object$coefficientNames, timeEffectCovariatesName)
+    }
+}
 
 #' @title finalizeSqlCyclopsData
 #'
