@@ -623,6 +623,22 @@ int cyclopsLoadDataX(Environment x,
                        covariateValue, replace, append, forceSparse);
 }
 
+// [[Rcpp::export(".loadCyclopsDataStratTimeEffects")]]
+std::vector<std::string> cyclopsLoadDataStratTimeEffects(Environment x,
+                                    const std::vector<double>& stratumId,
+                                    const std::vector<double>& rowId,
+                                    const std::vector<double>& subjectId,
+                                    const std::vector<double>& timeEffectCovariateId) {
+
+    using namespace bsccs;
+    XPtr<AbstractModelData> data = parseEnvironmentForPtr(x);
+
+    return data->loadStratTimeEffects(reinterpret_cast<const std::vector<int64_t>&>(stratumId),
+		    reinterpret_cast<const std::vector<int64_t>&>(rowId),
+		    reinterpret_cast<const std::vector<int64_t>&>(subjectId),
+		    reinterpret_cast<const std::vector<int64_t>&>(timeEffectCovariateId));
+}
+
 // NOTE:  IdType does not get exported into RcppExports, so hard-coded here
 // TODO Could use SEXP signature and cast in function
 
