@@ -15,8 +15,9 @@
 #include <thread>
 #include <complex>
 
-// #define CYCLOPS_DEBUG_TIMING
-// #define CYCLOPS_DEBUG_TIMING_LOW
+//#define CYCLOPS_DEBUG_TIMING
+//#define CYCLOPS_DEBUG_TIMING_GPU_COX
+//#define CYCLOPS_DEBUG_TIMING_LOW
 
 #ifdef CYCLOPS_DEBUG_TIMING
     #include "Timing.h"
@@ -39,6 +40,7 @@
 
 #define Fraction std::complex
 
+
 namespace bsccs {
 
 template <typename RealType>
@@ -55,8 +57,11 @@ protected:
     RealVector hXBeta;
     RealVector offsExpXBeta;
     RealVector denomPid;
+    RealVector denomPid2;
     RealVector numerPid;
     RealVector numerPid2;
+    RealVector numerPid3;
+    RealVector numerPid4;
 
     RealVector hNWeight;
     RealVector hKWeight;
@@ -83,7 +88,7 @@ public:
 			const std::vector<bool>& fixBeta,
 			bool useWeights);
 
-	AbstractModelSpecifics* clone() const;
+	AbstractModelSpecifics* clone(ComputeDeviceArguments computeDevice) const;
 
 	virtual const std::vector<double> getXBeta();
 
@@ -97,8 +102,26 @@ public:
 
 	virtual void computeXBeta(double* beta, bool useWeights);
 
+	// GPU
+
+	virtual void turnOnSyncCV(int foldToCompute) {};
+
+	virtual void turnOffSyncCV(void) {};
+
+	virtual void turnOnStreamCV(int foldToCompute) {};
+
+	virtual bool isGPU() {return false;};
+
+	virtual bool isCUDA() {return false;};
+
+	virtual double getPredictiveLogLikelihood(double* weights, int cvIndex) {return 0;};
+
 	//virtual double getGradientObjective();
 
+	virtual void setPidForAccumulation(const double* weights);
+
+	virtual void getOriginalPid();
+
 	virtual void deviceInitialization();
 
 	void initialize(
@@ -146,8 +169,11 @@ protected:
 
     using BaseModel::offsExpXBeta;
     using BaseModel::denomPid;
+    using BaseModel::denomPid2;
     using BaseModel::numerPid;
     using BaseModel::numerPid2;
+    using BaseModel::numerPid3;
+    using BaseModel::numerPid4;
 
     // RealVector offsExpXBeta;
     // RealVector denomPid;
@@ -207,10 +233,12 @@ protected:
 	//void setWeights(double* inWeights, bool useCrossValidation);
 	void setWeights(double* inWeights, double *cenWeights, bool useCrossValidation);
 
+	void setWeights(double* inWeights, bool useCrossValidation, int index) {};
+
 	void doSortPid(bool useCrossValidation);
 
 	template <typename AnyRealType>
-	void setPidForAccumulation(const AnyRealType* weights);
+	void setPidForAccumulationImpl(const AnyRealType* weights);
 
 	void setupSparseIndices(const int max);
 
@@ -222,10 +250,36 @@ protected:
 
 	using Storage<RealType>::hNWeight;
 	using Storage<RealType>::hKWeight;
-    using Storage<RealType>::hYWeight; // ESK: Added
+	using Storage<RealType>::hYWeight; // ESK: Added
+	std::vector<double> hYWeightDouble;
+	std::vector<int> hNtoK;
+
+	RealVector norm;
+
+//	 std::vector<RealType> hNWeight;
+//	 std::vector<RealType> hKWeight;
+
+	void computeXjY(bool useCrossValidation);
+
+	void computeXjX(bool useCrossValidation);
+
+	void computeNtoKIndices(bool useCrossValidation);
+
+	template <class OutType, class InType>
+	void incrementByGroup(OutType* values, int* groups, int k, InType inc) {
+		values[BaseModel::getGroup(groups, k)] += inc; // TODO delegate to BaseModel (different in tied-models)
+	}
+
+	RealType observationCount(RealType yi) {
+		return BaseModel::observationCount(yi);
+	}
+
+
+	std::vector<double> getGradientObjectives(void) {
+		std::vector<double> result;
+		return result;
+	}
 
-	// std::vector<RealType> hNWeight;
-	// std::vector<RealType> hKWeight;
 
 #ifdef CYCLOPS_DEBUG_TIMING
 	//	std::vector<double> duration;
@@ -262,11 +316,6 @@ private:
 	template <class IteratorType, class Weights>
 	void updateXBetaImpl(RealType delta, int index);
 
-	template <class OutType, class InType>
-	void incrementByGroup(OutType* values, int* groups, int k, InType inc) {
-	    values[BaseModel::getGroup(groups, k)] += inc; // TODO delegate to BaseModel (different in tied-models)
-	}
-
 	template <typename IteratorTypeOne, class Weights>
 	void dispatchFisherInformation(int indexOne, int indexTwo, double *oinfo, Weights w);
 
@@ -276,12 +325,6 @@ private:
 	template<class IteratorType>
 	SparseIterator<RealType> getSubjectSpecificHessianIterator(int index);
 
-	void computeXjY(bool useCrossValidation);
-
-	void computeXjX(bool useCrossValidation);
-
-	void computeNtoKIndices(bool useCrossValidation);
-
 	void initializeMmXt();
 
 	void initializeMM(
@@ -294,10 +337,6 @@ private:
 	template <class InteratorType>
 	void incrementNormsImpl(int index);
 
-	std::vector<int> hNtoK;
-
-	RealVector norm;
-
 	struct WeightedOperation {
 		const static bool isWeighted = true;
 	} weighted;
@@ -434,11 +473,15 @@ public:
 
 	const static bool hasTwoNumeratorTerms = true;
 
+	const static bool efron = false;
+
 	const static bool exactCLR = false;
 
     const static bool isTwoWayScan = false; // ESK: Added
 
-    const static bool isSurvivalModel = false; //ESK: Added
+	const static bool isSurvivalModel = false; //ESK: Added
+
+	const static bool isScanByKey = false;
 
     template <class XType, typename RealType>
 	RealType gradientNumeratorContrib(XType x, RealType predictor, RealType xBeta, RealType y) { // TODO Make static again?
@@ -446,8 +489,8 @@ public:
 		return predictor * x;
 	}
 
-    template <typename RealType>
-    RealType logLikeNumeratorContrib(int yi, RealType xBetai) {
+	template <typename RealType>
+	RealType logLikeNumeratorContrib(int yi, RealType xBetai) {
         return yi * xBetai;
 	}
 
@@ -455,6 +498,7 @@ public:
 	static RealType gradientNumerator2Contrib(XType x, RealType predictor) {
 		return predictor * x * x;
 	}
+
 };
 
 //ESK: Create new structure for survival data (added isSurvivalModel & new logLikeNumeratorContrib)
@@ -770,7 +814,7 @@ public:
 			RealType* gradient, RealType* hessian,
 			RealType numer, RealType numer2, RealType denom,
 			RealType nEvents,
-			RealType x, RealType xBeta, RealType y) {
+			RealType x, RealType xBeta, RealType y) { // TODO hacking x to provide numerPid
 
 		const RealType t = numer / denom;
 		const RealType g = nEvents * t; // Always use weights (number of events)
@@ -922,6 +966,99 @@ public:
 
 };
 
+template <typename RealType>
+struct EfronConditionalLogisticRegression : public Storage<RealType>, GroupedData, GLMProjection, FixedPid, Survival<RealType> {
+public:
+
+	const static bool efron = true;
+
+    typedef typename Storage<RealType>::RealVector RealVector;
+    EfronConditionalLogisticRegression(const RealVector& y, const RealVector& offs)
+        : Storage<RealType>(y, offs) { }
+
+	const static bool precomputeHessian = false; // XjX
+	const static bool likelihoodHasFixedTerms = false;
+
+	RealType logLikeFixedTermsContrib(RealType yi, RealType offseti, RealType logoffseti) {
+        throw new std::logic_error("Not model-specific");
+		return static_cast<RealType>(0);
+	}
+
+	static RealType getDenomNullValue () { return static_cast<RealType>(0); }
+
+	RealType observationCount(RealType yi) {
+		return static_cast<RealType>(yi);
+	}
+
+	template <class IteratorType, class Weights>
+	void incrementGradientAndHessian(
+			const IteratorType& it,
+			Weights false_signature,
+			RealType* gradient, RealType* hessian,
+			RealType numer, RealType numer2, RealType denom,
+			RealType nEvents,
+			RealType x, RealType xBeta, RealType y) { // TODO hacking x, xBeta, y
+
+		const RealType numer3 = x;
+		const RealType numer4 = xBeta;
+		const RealType denom2 = y;
+
+		for (int j = 0; j < nEvents; j++) {
+			RealType n = numer - j / nEvents * numer3;
+			RealType d = denom - j / nEvents * denom2;
+			RealType t = n / d;
+			*gradient += t;
+			if (IteratorType::isIndicator) {
+				*hessian += t * (static_cast<RealType>(1.0) - t);
+			} else {
+				*hessian += (numer2 - j / nEvents * numer4) / d - t * t;
+			}
+		}
+	}
+
+	template <class IteratorType, class WeightOerationType>
+	inline Fraction<RealType> incrementGradientAndHessian(const Fraction<RealType>& lhs,
+	    RealType numerator, RealType numerator2, RealType denominator, RealType weight,
+	    RealType xBeta, RealType y) {
+
+    	const RealType g = numerator / denominator;
+    	const RealType gradient = weight * g; // Always use weights (number of events)
+
+        const RealType hessian =
+            (IteratorType::isIndicator) ?
+                gradient * (static_cast<RealType>(1.0) - g) :
+                weight * (numerator2 / denominator - g * g);
+
+        return { lhs.real() + gradient, lhs.imag() + hessian };
+    }
+
+	RealType getOffsExpXBeta(const RealType offs, const RealType xBeta) {
+        return std::exp(xBeta);
+	}
+
+	RealType getOffsExpXBeta(const RealType* offs, RealType xBeta, RealType y, int k) {
+		return std::exp(xBeta);
+	}
+
+	RealType logLikeDenominatorContrib(RealType ni, RealType denom) {
+		return ni * std::log(denom);
+	}
+
+	RealType logPredLikeContrib(RealType y, RealType weight, RealType xBeta, RealType denominator) {
+	    return y * weight * (xBeta - std::log(denominator));
+	}
+
+	RealType logPredLikeContrib(RealType ji, RealType weighti, RealType xBetai, const RealType* denoms,
+			const int* groups, int i) {
+		return ji * weighti * (xBetai - std::log(denoms[getGroup(groups, i)]));
+	}
+
+	RealType predictEstimate(RealType xBeta){
+		return xBeta;
+	}
+
+};
+
 template <typename RealType>
 struct TiedConditionalLogisticRegression : public Storage<RealType>, GroupedWithTiesData, GLMProjection, FixedPid, Survival<RealType> {
 public:
@@ -1109,6 +1246,8 @@ public:
 
     const static bool isTwoWayScan = false; // ESK: Added
 
+    const static bool isScanByKey = false;
+
 	static RealType getDenomNullValue () { return static_cast<RealType>(0); }
 
 	RealType setIndependentDenominator(RealType expXBeta) {
@@ -1121,7 +1260,7 @@ public:
 		return static_cast<RealType>(yi);
 	}
 
-	template <class IteratorType, class Weights>
+    template <class IteratorType, class Weights>		
     inline void incrementGradientAndHessian( // TODO Make static again?
             const IteratorType& it,
             Weights false_signature,
@@ -1213,6 +1352,8 @@ public:
 
     const static bool isTwoWayScan = false; // ESK: Added (change this once FINE_GRAY works)
 
+    const static bool isScanByKey = false;
+
     static RealType getDenomNullValue () { return static_cast<RealType>(0); }
 
     bool resetAccumulators(int* pid, int k, int currentPid) {
@@ -1317,6 +1458,16 @@ public:
 	}
 };
 
+template <typename RealType>
+struct TimeVaryingCoxProportionalHazards : public BreslowTiedCoxProportionalHazards<RealType> {
+public:
+    typedef typename Storage<RealType>::RealVector RealVector;
+    TimeVaryingCoxProportionalHazards(const RealVector& y, const RealVector& offs)
+        : BreslowTiedCoxProportionalHazards<RealType>(y, offs) { }
+
+    const static bool isScanByKey = true;
+};
+
 // ESK: Add Fine-Gray structure
 template <typename RealType>
 struct BreslowTiedFineGray: public Storage<RealType>, OrderedWithTiesData, SurvivalProjection, SortedPid, NoFixedLikelihoodTerms, Survival<RealType> {
@@ -1329,6 +1480,8 @@ public:
 
     const static bool isTwoWayScan = true; // ESK: Added
 
+    const static bool isScanByKey = false;
+
     static RealType getDenomNullValue () { return static_cast<RealType>(0); }
 
     bool resetAccumulators(int* pid, int k, int currentPid) {
@@ -1430,12 +1583,16 @@ public:
 
 	const static bool hasTwoNumeratorTerms = false;
 
+	const static bool efron = false;
+
 	const static bool exactCLR = false;
 
     const static bool isTwoWayScan = false; // ESK: Added
 
     const static bool isSurvivalModel = false; //ESK: Added
 
+    const static bool isScanByKey = false;
+
     static RealType getDenomNullValue () { return static_cast<RealType>(0); }
 
 	RealType observationCount(RealType yi) {
