@@ -97,3 +97,31 @@ start, length, event, x1, x2
     expect_equal(coef(fit)["x1"], argMax)
 })
 
+test_that("Check adapative profiling likelihood, other covariate is perfect predictor", {
+
+    test <- read.table(header=T, sep = ",", text = "
+start, length, event, x1, x2
+0, 4,  1,0,1
+0, 4,1,2,1
+0, 4,  0,0,0
+0, 4,1,0,1
+0, 4,  1,1,1
+0, 4,0,1,0
+0, 4,  1,1,1
+")
+    data <- createCyclopsData(event ~ x1 + x2, data = test,
+                              modelType = "cpr")
+
+    # Recreating combination of circumstances that lead to infinite log likelihood
+    # Note: infinite variance would have been selected by cross-validation if we had enough data:
+    fit <- fitCyclopsModel(data,
+                           prior = createPrior("laplace", variance = Inf, exclude = "x1"))
+    expect_warning(getCyclopsProfileLogLikelihood(fit, "x1", bounds = c(0, 2), initialGridSize = 10),
+                   "Failing to compute likelihood at entire initial grid")
+
+    # This just produces NaN likelihood:
+    fit <- fitCyclopsModel(data)
+    expect_warning(getCyclopsProfileLogLikelihood(fit, "x1", bounds = c(0, 2), initialGridSize = 10),
+                   "Failing to compute likelihood at entire initial grid")
+})
+
